@@ -28,21 +28,28 @@
 #include "common/common.h"
 #include "audio/filter/af.h"
 
-int af_fmt2bits(int format)
+int af_fmt2bps(int format)
 {
-    if (AF_FORMAT_IS_AC3(format)) return 16;
-    if (format == AF_FORMAT_UNKNOWN)
-        return 0;
     switch (format & AF_FORMAT_BITS_MASK) {
-    case AF_FORMAT_8BIT:  return 8;
-    case AF_FORMAT_16BIT: return 16;
-    case AF_FORMAT_24BIT: return 24;
-    case AF_FORMAT_32BIT: return 32;
-    case AF_FORMAT_64BIT: return 64;
+    case AF_FORMAT_8BIT:  return 1;
+    case AF_FORMAT_16BIT: return 2;
+    case AF_FORMAT_24BIT: return 3;
+    case AF_FORMAT_32BIT: return 4;
+    case AF_FORMAT_64BIT: return 8;
     }
     return 0;
 }
 
+int af_fmt2bits(int format)
+{
+    return af_fmt2bps(format) * 8;
+}
+
+bool af_fmt_is_float(int format)
+{
+    return !!(format & AF_FORMAT_F);
+}
+
 static int bits_to_mask(int bits)
 {
     switch (bits) {
@@ -57,7 +64,7 @@ static int bits_to_mask(int bits)
 
 int af_fmt_change_bits(int format, int bits)
 {
-    if (!af_fmt_is_valid(format) || (format & AF_FORMAT_SPECIAL_MASK))
+    if (!af_fmt_is_valid(format))
         return 0;
     int mask = bits_to_mask(bits);
     format = (format & ~AF_FORMAT_BITS_MASK) | mask;
@@ -104,35 +111,31 @@ bool af_fmt_is_planar(int format)
     return !!(format & AF_FORMAT_PLANAR);
 }
 
-#define FMT(string, id)                                 \
-    {string,            id},
-
-#define FMT_ENDIAN(string, id)                          \
-    {string,            id},                            \
-    {string "le",       MP_CONCAT(id, _LE)},            \
-    {string "be",       MP_CONCAT(id, _BE)},            \
-
 const struct af_fmt_entry af_fmtstr_table[] = {
-    FMT("mpeg2",                AF_FORMAT_MPEG2)
-    FMT_ENDIAN("ac3",           AF_FORMAT_AC3)
-    FMT_ENDIAN("iec61937",      AF_FORMAT_IEC61937)
+    {"u8",          AF_FORMAT_U8},
+    {"s8",          AF_FORMAT_S8},
+    {"u16",         AF_FORMAT_U16},
+    {"s16",         AF_FORMAT_S16},
+    {"u24",         AF_FORMAT_U24},
+    {"s24",         AF_FORMAT_S24},
+    {"u32",         AF_FORMAT_U32},
+    {"s32",         AF_FORMAT_S32},
+    {"float",       AF_FORMAT_FLOAT},
+    {"double",      AF_FORMAT_DOUBLE},
 
-    FMT("u8",                   AF_FORMAT_U8)
-    FMT("s8",                   AF_FORMAT_S8)
-    FMT_ENDIAN("u16",           AF_FORMAT_U16)
-    FMT_ENDIAN("s16",           AF_FORMAT_S16)
-    FMT_ENDIAN("u24",           AF_FORMAT_U24)
-    FMT_ENDIAN("s24",           AF_FORMAT_S24)
-    FMT_ENDIAN("u32",           AF_FORMAT_U32)
-    FMT_ENDIAN("s32",           AF_FORMAT_S32)
-    FMT_ENDIAN("float",         AF_FORMAT_FLOAT)
-    FMT_ENDIAN("double",        AF_FORMAT_DOUBLE)
+    {"u8p",         AF_FORMAT_U8P},
+    {"s16p",        AF_FORMAT_S16P},
+    {"s32p",        AF_FORMAT_S32P},
+    {"floatp",      AF_FORMAT_FLOATP},
+    {"doublep",     AF_FORMAT_DOUBLEP},
 
-    FMT("u8p",                  AF_FORMAT_U8P)
-    FMT("s16p",                 AF_FORMAT_S16P)
-    FMT("s32p",                 AF_FORMAT_S32P)
-    FMT("floatp",               AF_FORMAT_FLOATP)
-    FMT("doublep",              AF_FORMAT_DOUBLEP)
+    {"spdif-aac",   AF_FORMAT_S_AAC},
+    {"spdif-ac3",   AF_FORMAT_S_AC3},
+    {"spdif-dts",   AF_FORMAT_S_DTS},
+    {"spdif-dtshd", AF_FORMAT_S_DTSHD},
+    {"spdif-eac3",  AF_FORMAT_S_EAC3},
+    {"spdif-mp3",   AF_FORMAT_S_MP3},
+    {"spdif-truehd",AF_FORMAT_S_TRUEHD},
 
     {0}
 };
@@ -159,7 +162,7 @@ const char *af_fmt_to_str(int format)
 int af_fmt_seconds_to_bytes(int format, float seconds, int channels, int samplerate)
 {
     assert(!af_fmt_is_planar(format));
-    int bps      = (af_fmt2bits(format) / 8);
+    int bps      = af_fmt2bps(format);
     int framelen = channels * bps;
     int bytes    = seconds  * bps * samplerate;
     if (bytes % framelen)
@@ -194,22 +197,17 @@ int af_format_conversion_score(int dst_format, int src_format)
         return INT_MIN;
     if (dst_format == src_format)
         return 1024;
-    // Just endian swapping (separate because it works for special formats)
-    if ((dst_format & ~AF_FORMAT_END_MASK) == (src_format & ~AF_FORMAT_END_MASK))
-        return 1024 - 2;
     // Can't be normally converted
     if (AF_FORMAT_IS_SPECIAL(dst_format) || AF_FORMAT_IS_SPECIAL(src_format))
         return INT_MIN;
     int score = 1024;
     if (FMT_DIFF(AF_FORMAT_INTERLEAVING_MASK, dst_format, src_format))
         score -= 1;     // has to (de-)planarize
-    if (FMT_DIFF(AF_FORMAT_END_MASK, dst_format, src_format))
-        score -= 2;     // has to swap endian
     if (FMT_DIFF(AF_FORMAT_SIGN_MASK, dst_format, src_format))
         score -= 4;     // has to swap sign
-    if (FMT_DIFF(AF_FORMAT_POINT_MASK, dst_format, src_format)) {
+    if (FMT_DIFF(AF_FORMAT_TYPE_MASK, dst_format, src_format)) {
         int dst_bits = dst_format & AF_FORMAT_BITS_MASK;
-        if ((dst_format & AF_FORMAT_POINT_MASK) == AF_FORMAT_F) {
+        if ((dst_format & AF_FORMAT_TYPE_MASK) == AF_FORMAT_F) {
             // For int->float, always prefer 32 bit float.
             score -= dst_bits == AF_FORMAT_32BIT ? 8 : 0;
         } else {
@@ -225,7 +223,7 @@ int af_format_conversion_score(int dst_format, int src_format)
         }
     }
     // Consider this the worst case.
-    if (FMT_DIFF(AF_FORMAT_POINT_MASK, dst_format, src_format))
+    if (FMT_DIFF(AF_FORMAT_TYPE_MASK, dst_format, src_format))
         score -= 2048;  // has to convert float<->int
     return score;
 }
