@@ -46,11 +46,25 @@ enum aocontrol {
 // data might be written again, instead of closing the AO.
 #define AOPLAY_FINAL_CHUNK 1
 
+enum {
+    AO_EVENT_RELOAD = 1,
+};
+
 typedef struct ao_control_vol {
     float left;
     float right;
 } ao_control_vol_t;
 
+struct ao_device_desc {
+    const char *name;   // symbolic name; will be set on ao->device
+    const char *desc;   // verbose human readable name
+};
+
+struct ao_device_list {
+    struct ao_device_desc *devices;
+    int num_devices;
+};
+
 struct ao;
 struct mpv_global;
 struct input_ctx;
@@ -65,6 +79,7 @@ void ao_uninit(struct ao *ao);
 void ao_get_format(struct ao *ao, struct mp_audio *format);
 const char *ao_get_name(struct ao *ao);
 const char *ao_get_description(struct ao *ao);
+const char *ao_get_detected_device(struct ao *ao);
 bool ao_untimed(struct ao *ao);
 int ao_play(struct ao *ao, void **data, int samples, int flags);
 int ao_control(struct ao *ao, enum aocontrol cmd, void *arg);
@@ -75,5 +90,10 @@ void ao_pause(struct ao *ao);
 void ao_resume(struct ao *ao);
 void ao_drain(struct ao *ao);
 bool ao_eof_reached(struct ao *ao);
+int ao_query_and_reset_events(struct ao *ao, int events);
+void ao_request_reload(struct ao *ao);
+
+struct ao_device_list *ao_get_device_list(struct mpv_global *global);
+void ao_print_devices(struct mpv_global *global, struct mp_log *log);
 
 #endif /* MPLAYER_AUDIO_OUT_H */
