@@ -156,6 +156,11 @@ static int init(struct ao *ao)
     OSStatus err = ca_select_device(ao, ao->device, &p->device);
     CHECK_CA_ERROR("failed to select device");
 
+    char *uid;
+    err = CA_GET_STR(p->device, kAudioDevicePropertyDeviceUID, &uid);
+    CHECK_CA_ERROR("failed to get device UID");
+    ao->detected_device = talloc_steal(ao, uid);
+
     if (!init_chmap(ao))
         goto coreaudio_error;
 
