@@ -20,9 +20,6 @@
 #ifndef MP_AO_WASAPI_H_
 #define MP_AO_WASAPI_H_
 
-#define COBJMACROS 1
-#define _WIN32_WINNT 0x600
-
 #include <audioclient.h>
 #include <audiopolicy.h>
 #include <mmdeviceapi.h>
@@ -30,12 +27,26 @@
 
 #include "osdep/atomics.h"
 
+typedef struct change_notify {
+    IMMNotificationClient client; /* this must be first in the structure! */
+    LPWSTR monitored; /* Monitored device */
+    struct ao *ao;
+} change_notify;
+
+HRESULT wasapi_change_init(struct ao* ao);
+void wasapi_change_uninit(struct ao* ao);
+
+#define EXIT_ON_ERROR(hres)  \
+              do { if (FAILED(hres)) { goto exit_label; } } while(0)
+#define SAFE_RELEASE(unk, release) \
+              do { if ((unk) != NULL) { release; (unk) = NULL; } } while(0)
+
 typedef struct wasapi_state {
     struct mp_log *log;
     HANDLE threadLoop;
 
     /* Init phase */
-    int init_ret;
+    HRESULT init_ret;
     HANDLE init_done;
     int share_mode;
 
@@ -49,10 +60,6 @@ typedef struct wasapi_state {
 
     /* Buffers */
     size_t buffer_block_size; /* Size of each block in bytes */
-    REFERENCE_TIME
-        minRequestedDuration; /* minimum wasapi buffer block size, in 100-nanosecond units */
-    REFERENCE_TIME
-        defaultRequestedDuration; /* default wasapi default block size, in 100-nanosecond units */
     UINT32 bufferFrameCount; /* wasapi buffer block size, number of frames, frame size at format.nBlockAlign */
 
     /* WASAPI handles, owned by other thread */
@@ -62,6 +69,8 @@ typedef struct wasapi_state {
     ISimpleAudioVolume *pAudioVolume;
     IAudioEndpointVolume *pEndpointVolume;
     IAudioSessionControl *pSessionControl;
+    IMMDeviceEnumerator *pEnumerator;
+
     HANDLE hFeed; /* wasapi event */
     HANDLE hForceFeed; /* forces writing a buffer (e.g. before audio_resume) */
     HANDLE hFeedDone; /* set only after a hForceFeed */
@@ -94,12 +103,17 @@ typedef struct wasapi_state {
     int opt_list;
     char *opt_device;
 
-    /* We still need to support XP, don't use these functions directly, blob owned by main thread */
+    /* Don't use these functions directly in case
+       they are unimplemented for some reason.
+       (XP shouldn't be an issue since it doesn't support wasapi, maybe wine?)
+       Blob is owned by the main thread */
     struct {
         HMODULE hAvrt;
         HANDLE (WINAPI *pAvSetMmThreadCharacteristicsW)(LPCWSTR, LPDWORD);
         WINBOOL (WINAPI *pAvRevertMmThreadCharacteristics)(HANDLE);
     } VistaBlob;
+
+    change_notify change;
 } wasapi_state;
 
 #endif
