@@ -30,7 +30,8 @@
 struct playlist_entry *playlist_entry_new(const char *filename)
 {
     struct playlist_entry *e = talloc_zero(NULL, struct playlist_entry);
-    e->filename = talloc_strdup(e, filename);
+    char *local_filename = mp_file_url_to_filename(e, bstr0(filename));
+    e->filename = local_filename ? local_filename : talloc_strdup(e, filename);
     return e;
 }
 
@@ -105,10 +106,18 @@ static void playlist_unlink(struct playlist *pl, struct playlist_entry *entry)
     entry->pl = NULL;
 }
 
+void playlist_entry_unref(struct playlist_entry *e)
+{
+    e->reserved--;
+    if (e->reserved < 0)
+        talloc_free(e);
+}
+
 void playlist_remove(struct playlist *pl, struct playlist_entry *entry)
 {
     playlist_unlink(pl, entry);
-    talloc_free(entry);
+    entry->removed = true;
+    playlist_entry_unref(entry);
 }
 
 void playlist_clear(struct playlist *pl)
@@ -215,6 +224,15 @@ void playlist_transfer_entries(struct playlist *pl, struct playlist *source_pl)
     }
 }
 
+void playlist_append_entries(struct playlist *pl, struct playlist *source_pl)
+{
+    while (source_pl->first) {
+        struct playlist_entry *e = source_pl->first;
+        playlist_unlink(source_pl, e);
+        playlist_add(pl, e);
+    }
+}
+
 // Return number of entries between list start and e.
 // Return -1 if e is not on the list, or if e is NULL.
 int playlist_entry_to_index(struct playlist *pl, struct playlist_entry *e)
@@ -277,9 +295,6 @@ struct playlist *playlist_parse_file(const char *file, struct mpv_global *global
     if (ret && !ret->first)
         mp_warn(log, "Warning: empty playlist\n");
 
-    if (ret)
-        playlist_add_base_path(ret, mp_dirname(file));
-
     talloc_free(log);
     return ret;
 }
