@@ -55,6 +55,8 @@ enum demux_ctrl {
     DEMUXER_CTRL_IDENTIFY_PROGRAM,
     DEMUXER_CTRL_STREAM_CTRL,
     DEMUXER_CTRL_GET_READER_STATE,
+    DEMUXER_CTRL_GET_NAV_EVENT,
+    DEMUXER_CTRL_GET_BITRATE_STATS, // double[STREAM_TYPE_COUNT]
 };
 
 struct demux_ctrl_reader_state {
@@ -69,11 +71,12 @@ struct demux_ctrl_stream_ctrl {
     int res;
 };
 
-#define SEEK_ABSOLUTE (1 << 0)
-#define SEEK_FACTOR   (1 << 1)
-#define SEEK_FORWARD  (1 << 2)
-#define SEEK_BACKWARD (1 << 3)
-#define SEEK_SUBPREROLL (1 << 4)
+#define SEEK_ABSOLUTE (1 << 0)      // argument is a timestamp
+#define SEEK_FACTOR   (1 << 1)      // argument is in range [0,1]
+#define SEEK_FORWARD  (1 << 2)      // prefer later time if not exact
+#define SEEK_BACKWARD (1 << 3)      // prefer earlier time if not exact
+#define SEEK_SUBPREROLL (1 << 4)    // try to get more subtitle packets
+#define SEEK_HR       (1 << 5)      // hr-seek (this is a weak hint only)
 
 // Strictness of the demuxer open format check.
 // demux.c will try by default: NORMAL, UNSAFE (in this order)
@@ -123,7 +126,7 @@ typedef struct demuxer_desc {
 typedef struct demux_chapter
 {
     int original_index;
-    uint64_t start, end;
+    double pts;
     char *name;
     struct mp_tags *metadata;
     uint64_t demuxer_id; // for mapping to internal demuxer data structures
@@ -182,10 +185,14 @@ typedef struct demuxer {
     int64_t filepos;  // input stream current pos.
     char *filename;  // same as stream->url
     enum demuxer_type type;
-    int seekable; // flag
+    bool seekable;
+    bool partially_seekable; // implies seekable=true
     double start_time;
     // File format allows PTS resets (even if the current file is without)
     bool ts_resets_possible;
+    // Send relative seek requests, instead of SEEK_ABSOLUTE or SEEK_FACTOR.
+    // This is only done if the user explicitly uses a relative seek.
+    bool rel_seeks;
 
     // Bitmask of DEMUX_EVENT_*
     int events;
@@ -254,7 +261,7 @@ void demux_stop_thread(struct demuxer *demuxer);
 void demux_set_wakeup_cb(struct demuxer *demuxer, void (*cb)(void *ctx), void *ctx);
 
 void demux_flush(struct demuxer *demuxer);
-int demux_seek(struct demuxer *demuxer, float rel_seek_secs, int flags);
+int demux_seek(struct demuxer *demuxer, double rel_seek_secs, int flags);
 
 int demux_control(struct demuxer *demuxer, int cmd, void *arg);
 
@@ -268,8 +275,8 @@ void demuxer_help(struct mp_log *log);
 
 int demuxer_add_attachment(struct demuxer *demuxer, struct bstr name,
                            struct bstr type, struct bstr data);
-int demuxer_add_chapter(struct demuxer *demuxer, struct bstr name,
-                        uint64_t start, uint64_t end, uint64_t demuxer_id);
+int demuxer_add_chapter(demuxer_t *demuxer, struct bstr name,
+                        double pts, uint64_t demuxer_id);
 
 double demuxer_get_time_length(struct demuxer *demuxer);
 
@@ -284,6 +291,8 @@ void demux_update(demuxer_t *demuxer);
 struct sh_stream *demuxer_stream_by_demuxer_id(struct demuxer *d,
                                                enum stream_type t, int id);
 
+struct demux_chapter *demux_copy_chapter_data(struct demux_chapter *c, int num);
+
 bool demux_matroska_uid_cmp(struct matroska_segment_uid *a,
                             struct matroska_segment_uid *b);
 
