@@ -262,34 +262,38 @@ int ebml_resync_cluster(struct mp_log *log, stream_t *s)
 struct generic;
 #define generic_struct struct generic
 
-static uint32_t ebml_parse_id(uint8_t *data, int *length)
+static uint32_t ebml_parse_id(uint8_t *data, size_t data_len, int *length)
 {
+    *length = -1;
+    uint8_t *end = data + data_len;
+    if (data == end)
+        return EBML_ID_INVALID;
     int len = 1;
     uint32_t id = *data++;
     for (int len_mask = 0x80; !(id & len_mask); len_mask >>= 1) {
         len++;
-        if (len > 4) {
-            *length = -1;
+        if (len > 4)
             return EBML_ID_INVALID;
-        }
     }
     *length = len;
-    while (--len)
+    while (--len && data < end)
         id = (id << 8) | *data++;
     return id;
 }
 
-static uint64_t parse_vlen(uint8_t *data, int *length, bool is_length)
+static uint64_t ebml_parse_length(uint8_t *data, size_t data_len, int *length)
 {
+    *length = -1;
+    uint8_t *end = data + data_len;
+    if (data == end)
+        return -1;
     uint64_t r = *data++;
     int len = 1;
     int len_mask;
     for (len_mask = 0x80; !(r & len_mask); len_mask >>= 1) {
         len++;
-        if (len > 8) {
-            *length = -1;
+        if (len > 8)
             return -1;
-        }
     }
     r &= len_mask - 1;
 
@@ -297,25 +301,20 @@ static uint64_t parse_vlen(uint8_t *data, int *length, bool is_length)
     if (r == len_mask - 1)
         num_allones++;
     for (int i = 1; i < len; i++) {
+        if (data == end)
+            return -1;
         if (*data == 255)
             num_allones++;
         r = (r << 8) | *data++;
     }
-    if (is_length && num_allones == len) {
-        // According to Matroska specs this means "unknown length"
-        // Could be supported if there are any actual files using it
-        *length = -1;
+    // According to Matroska specs this means "unknown length"
+    // Could be supported if there are any actual files using it
+    if (num_allones == len)
         return -1;
-    }
     *length = len;
     return r;
 }
 
-static uint64_t ebml_parse_length(uint8_t *data, int *length)
-{
-    return parse_vlen(data, length, true);
-}
-
 static uint64_t ebml_parse_uint(uint8_t *data, int length)
 {
     assert(length >= 1 && length <= 8);
@@ -363,7 +362,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
     while (p < end) {
         uint8_t *startp = p;
         int len;
-        uint32_t id = ebml_parse_id(p, &len);
+        uint32_t id = ebml_parse_id(p, end - p, &len);
         if (len > end - p)
             goto past_end_error;
         if (len < 0) {
@@ -371,7 +370,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
             goto other_error;
         }
         p += len;
-        uint64_t length = ebml_parse_length(p, &len);
+        uint64_t length = ebml_parse_length(p, end - p, &len);
         if (len > end - p)
             goto past_end_error;
         if (len < 0) {
@@ -458,11 +457,17 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
 
     while (data < end) {
         int len;
-        uint32_t id = ebml_parse_id(data, &len);
-        assert(len >= 0 && len <= end - data);
+        uint32_t id = ebml_parse_id(data, end - data, &len);
+        if (len < 0 || len > end - data) {
+            MP_DBG(ctx, "Error parsing subelement\n");
+            break;
+        }
         data += len;
-        uint64_t length = ebml_parse_length(data, &len);
-        assert(len >= 0 && len <= end - data);
+        uint64_t length = ebml_parse_length(data, end - data, &len);
+        if (len < 0 || len > end - data) {
+            MP_DBG(ctx, "Error parsing subelement length\n");
+            break;
+        }
         data += len;
         if (length > end - data) {
             // Try to parse what is possible from inside this partial element
@@ -582,7 +587,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         case EBML_TYPE_EBML_ID:;
             uint32_t *idptr;
             GETPTR(idptr, uint32_t);
-            *idptr = ebml_parse_id(data, &len);
+            *idptr = ebml_parse_id(data, end - data, &len);
             if (len != length) {
                 MP_DBG(ctx, "ebml_id broken value\n");
                 goto error;
@@ -614,7 +619,7 @@ int ebml_read_element(struct stream *s, struct ebml_parse_ctx *ctx,
         MP_MSG(ctx, msglevel, "Refusing to read element over 100 MB in size\n");
         return -1;
     }
-    ctx->talloc_ctx = talloc_size(NULL, length + 8);
+    ctx->talloc_ctx = talloc_size(NULL, length);
     int read_len = stream_read(s, ctx->talloc_ctx, length);
     if (read_len < length)
         MP_MSG(ctx, msglevel, "Unexpected end of file - partial or corrupt file?\n");
