@@ -36,6 +36,8 @@ struct sh_stream {
     // Demuxer/format specific ID. Corresponds to the stream IDs as encoded in
     // some file formats (e.g. MPEG), or an index chosen by demux.c.
     int demuxer_id;
+    // FFmpeg stream index (AVFormatContext.streams[index]), or equivalent.
+    int ff_index;
     // One of these is non-NULL, the others are NULL, depending on the stream
     // type.
     struct sh_audio *audio;
@@ -48,8 +50,7 @@ struct sh_stream {
     // Usually a FourCC, exact meaning depends on codec.
     unsigned int format;
 
-    // Codec specific header data (set by demux_lavf.c)
-    // Other demuxers use sh_audio->wf and sh_video->bih instead.
+    // Codec specific header data (set by demux_lavf.c only)
     struct AVCodecContext *lav_headers;
 
     char *title;
