@@ -65,6 +65,10 @@ extern const m_option_type_t m_option_type_size_box;
 extern const m_option_type_t m_option_type_chmap;
 extern const m_option_type_t m_option_type_node;
 
+// Used internally by m_config.c
+extern const m_option_type_t m_option_type_alias;
+extern const m_option_type_t m_option_type_removed;
+
 // Callback used by m_option_type_print_fn options.
 typedef void (*m_opt_print_fn)(struct mp_log *log);
 
@@ -679,4 +683,13 @@ extern const char m_option_path_separator;
                        .type = &m_option_type_subconfig,        \
                        .priv = (void*)&subconf)
 
+// If "--name" was removed, but "--newname" has the same semantics.
+// It will be redirected, and a warning will be printed on first use.
+#define OPT_REPLACED(optname, newname) \
+    {.name = optname, .type = &m_option_type_alias, .priv = newname, .offset = -1}
+
+// "--name" doesn't exist, but inform the user about a replacement with msg.
+#define OPT_REMOVED(optname, msg) \
+    {.name = optname, .type = &m_option_type_removed, .priv = msg, .offset = -1}
+
 #endif /* MPLAYER_M_OPTION_H */
