@@ -25,6 +25,9 @@ struct input_ctx;
 
 void cocoa_put_key(int keycode);
 void cocoa_put_key_with_modifiers(int keycode, int modifiers);
+void cocoa_put_key_event(void *event);
+
+void cocoa_start_event_monitor(void);
 
 void cocoa_init_apple_remote(void);
 void cocoa_uninit_apple_remote(void);
