@@ -33,10 +33,23 @@ void property_print_help(struct mp_log *log);
 int mp_property_do(const char* name, int action, void* val,
                    struct MPContext *mpctx);
 
-const struct m_property *mp_get_property_list(void);
-int mp_find_property_index(const char *property);
-
 void mp_notify(struct MPContext *mpctx, int event, void *arg);
 void mp_notify_property(struct MPContext *mpctx, const char *property);
 
+int mp_get_property_id(const char *name);
+uint64_t mp_get_property_event_mask(const char *name);
+
+enum {
+    // Must start with the first unused positive value in enum mpv_event_id
+    // MPV_EVENT_* and MP_EVENT_* must not overlap.
+    INTERNAL_EVENT_BASE = 25,
+    MP_EVENT_CHANGE_ALL,
+    MP_EVENT_CACHE_UPDATE,
+    MP_EVENT_WIN_RESIZE,
+    MP_EVENT_WIN_STATE,
+};
+
+bool mp_hook_test_completion(struct MPContext *mpctx, char *type);
+void mp_hook_run(struct MPContext *mpctx, char *client, char *type);
+
 #endif /* MPLAYER_COMMAND_H */
