@@ -88,8 +88,7 @@ void update_fps(struct MPContext *mpctx)
 
 static void set_allowed_vo_formats(struct vf_chain *c, struct vo *vo)
 {
-    for (int fmt = IMGFMT_START; fmt < IMGFMT_END; fmt++)
-        c->allowed_output_formats[fmt - IMGFMT_START] = vo_query_format(vo, fmt);
+    vo_query_formats(vo, c->allowed_output_formats);
 }
 
 static int try_filter(struct MPContext *mpctx, struct mp_image_params params,
@@ -181,6 +180,8 @@ static void recreate_video_filters(struct MPContext *mpctx)
     d_video->vfilter->wakeup_callback = wakeup_playloop;
     d_video->vfilter->wakeup_callback_ctx = mpctx;
     d_video->vfilter->container_fps = d_video->fps;
+    vo_control(mpctx->video_out, VOCTRL_GET_DISPLAY_FPS,
+        &d_video->vfilter->display_fps);
 
     vf_append_filter_list(d_video->vfilter, opts->vf_settings);
 
@@ -217,6 +218,7 @@ void reset_video_state(struct MPContext *mpctx)
 
     mp_image_unrefp(&mpctx->next_frame[0]);
     mp_image_unrefp(&mpctx->next_frame[1]);
+    mp_image_unrefp(&mpctx->saved_frame);
 
     mpctx->delay = 0;
     mpctx->time_frame = 0;
@@ -266,10 +268,14 @@ int reinit_video_chain(struct MPContext *mpctx)
                sh->video->fps);
 
     //================== Init VIDEO (codec & libvo) ==========================
-    if (!opts->fixed_vo || !mpctx->video_out) {
-        mpctx->video_out = init_best_video_out(mpctx->global, mpctx->input,
-                                               mpctx->osd,
-                                               mpctx->encode_lavc_ctx);
+    if (!mpctx->video_out) {
+        struct vo_extra ex = {
+            .input_ctx = mpctx->input,
+            .osd = mpctx->osd,
+            .encode_lavc_ctx = mpctx->encode_lavc_ctx,
+            .opengl_cb_context = mpctx->gl_cb_ctx,
+        };
+        mpctx->video_out = init_best_video_out(mpctx->global, &ex);
         if (!mpctx->video_out) {
             MP_FATAL(mpctx, "Error opening/initializing "
                     "the selected video_out (-vo) device.\n");
@@ -524,7 +530,7 @@ static void adjust_sync(struct MPContext *mpctx, double v_pts, double frame_time
     if (mpctx->audio_status != STATUS_PLAYING)
         return;
 
-    double a_pts = written_audio_pts(mpctx) + mpctx->audio_delay - mpctx->delay;
+    double a_pts = written_audio_pts(mpctx) + opts->audio_delay - mpctx->delay;
     double av_delay = a_pts - v_pts;
 
     double change = av_delay * 0.1;
@@ -551,6 +557,8 @@ static bool have_new_frame(struct MPContext *mpctx)
 // returns VD_* code
 static int video_output_image(struct MPContext *mpctx, double endpts)
 {
+    bool hrseek = mpctx->hrseek_active && mpctx->video_status == STATUS_SYNCING;
+
     if (mpctx->d_video->header->attached_picture) {
         if (vo_has_frame(mpctx->video_out))
             return VD_EOF;
@@ -582,9 +590,8 @@ static int video_output_image(struct MPContext *mpctx, double endpts)
             frame_time = 0;
         }
         mpctx->video_next_pts = pts;
-        if (mpctx->d_audio)
-            mpctx->delay -= frame_time;
-        if (mpctx->video_status >= STATUS_READY) {
+        mpctx->delay -= frame_time;
+        if (mpctx->video_status >= STATUS_PLAYING) {
             mpctx->time_frame += frame_time / mpctx->opts->playback_speed;
             adjust_sync(mpctx, pts, frame_time);
         }
@@ -608,16 +615,18 @@ static int video_output_image(struct MPContext *mpctx, double endpts)
             add_frame_pts(mpctx, img->pts);
 
             bool drop = false;
-            bool hrseek = mpctx->hrseek_active
-                       && mpctx->video_status == STATUS_SYNCING;
-            if (hrseek && img->pts < mpctx->hrseek_pts - .005)
-                drop = true;
             if ((endpts != MP_NOPTS_VALUE && img->pts >= endpts) ||
                 mpctx->max_frames == 0)
             {
                 drop = true;
                 r = VD_EOF;
             }
+            if (!drop && hrseek && mpctx->hrseek_lastframe) {
+                mp_image_setrefp(&mpctx->saved_frame, img);
+                drop = true;
+            }
+            if (hrseek && img->pts < mpctx->hrseek_pts - .005)
+                drop = true;
             if (drop) {
                 talloc_free(img);
             } else {
@@ -630,6 +639,13 @@ static int video_output_image(struct MPContext *mpctx, double endpts)
     if (have_new_frame(mpctx) || (r <= 0 && mpctx->next_frame[0]))
         return VD_NEW_FRAME;
 
+    // Last-frame seek
+    if (r <= 0 && hrseek && mpctx->hrseek_lastframe && mpctx->saved_frame) {
+        mpctx->next_frame[1] = mpctx->saved_frame;
+        mpctx->saved_frame = NULL;
+        return VD_PROGRESS;
+    }
+
     return r;
 }
 
@@ -648,7 +664,11 @@ static void update_avsync_before_frame(struct MPContext *mpctx)
                !ao_untimed(mpctx->ao))
     {
         double buffered_audio = ao_get_delay(mpctx->ao);
-        MP_TRACE(mpctx, "audio delay=%f\n", buffered_audio);
+
+        double predicted = mpctx->delay / opts->playback_speed +
+                           mpctx->time_frame;
+        double difference = buffered_audio - predicted;
+        MP_STATS(mpctx, "value %f audio-diff", difference);
 
         if (opts->autosync) {
             /* Smooth reported playback position from AO by averaging
@@ -658,9 +678,6 @@ static void update_avsync_before_frame(struct MPContext *mpctx)
              * This is badly implemented; the behavior of the smoothing
              * now undesirably depends on how often this code runs
              * (mainly depends on video frame rate). */
-            float predicted = mpctx->delay / opts->playback_speed +
-                              mpctx->time_frame;
-            float difference = buffered_audio - predicted;
             buffered_audio = predicted + difference / opts->autosync;
         }
 
@@ -681,7 +698,8 @@ static void update_avsync_before_frame(struct MPContext *mpctx)
 // Update the A/V sync difference after a video frame has been shown.
 static void update_avsync_after_frame(struct MPContext *mpctx)
 {
-    mpctx->time_frame -= get_relative_time(mpctx);
+    struct MPOpts *opts = mpctx->opts;
+
     mpctx->last_av_difference = 0;
 
     if (mpctx->audio_status != STATUS_PLAYING ||
@@ -690,13 +708,12 @@ static void update_avsync_after_frame(struct MPContext *mpctx)
 
     double a_pos = playing_audio_pts(mpctx);
 
-    mpctx->last_av_difference = a_pos - mpctx->video_pts + mpctx->audio_delay;
+    mpctx->last_av_difference = a_pos - mpctx->video_pts + opts->audio_delay;
     if (mpctx->time_frame > 0)
-        mpctx->last_av_difference +=
-                mpctx->time_frame * mpctx->opts->playback_speed;
+        mpctx->last_av_difference += mpctx->time_frame * opts->playback_speed;
     if (a_pos == MP_NOPTS_VALUE || mpctx->video_pts == MP_NOPTS_VALUE)
         mpctx->last_av_difference = MP_NOPTS_VALUE;
-    if (mpctx->last_av_difference > 0.5 && !mpctx->drop_message_shown) {
+    if (fabs(mpctx->last_av_difference) > 0.5 && !mpctx->drop_message_shown) {
         MP_WARN(mpctx, "%s", av_desync_help_text);
         mpctx->drop_message_shown = true;
     }
@@ -759,9 +776,6 @@ void write_video(struct MPContext *mpctx, double endpts)
     if (mpctx->video_status > STATUS_PLAYING)
         mpctx->video_status = STATUS_PLAYING;
 
-    mpctx->time_frame -= get_relative_time(mpctx);
-    update_avsync_before_frame(mpctx);
-
     if (r != VD_NEW_FRAME) {
         mpctx->sleeptime = 0; // Decode more in next iteration.
         return;
@@ -788,9 +802,11 @@ void write_video(struct MPContext *mpctx, double endpts)
             goto error;
         }
         init_vo(mpctx);
-        mpctx->time_frame = 0; // display immediately
     }
 
+    mpctx->time_frame -= get_relative_time(mpctx);
+    update_avsync_before_frame(mpctx);
+
     double time_frame = MPMAX(mpctx->time_frame, -1);
     int64_t pts = mp_time_us() + (int64_t)(time_frame * 1e6);
 
@@ -823,6 +839,8 @@ void write_video(struct MPContext *mpctx, double endpts)
     mpctx->last_vo_pts = mpctx->video_pts;
     mpctx->playback_pts = mpctx->video_pts;
 
+    update_avsync_after_frame(mpctx);
+
     mpctx->osd_force_update = true;
     update_osd_msg(mpctx);
     update_subtitles(mpctx);
@@ -836,7 +854,6 @@ void write_video(struct MPContext *mpctx, double endpts)
         // After a seek, make sure to wait until the first frame is visible.
         vo_wait_frame(vo);
     }
-    update_avsync_after_frame(mpctx);
     screenshot_flip(mpctx);
 
     mp_notify(mpctx, MPV_EVENT_TICK, NULL);
