@@ -132,26 +132,25 @@ void mp_ass_configure(ASS_Renderer *priv, struct MPOpts *opts,
     ass_set_margins(priv, dim->mt, dim->mb, dim->ml, dim->mr);
 
     int set_use_margins = 0;
-#if LIBASS_VERSION >= 0x01010000
     int set_sub_pos = 0;
-#endif
     float set_line_spacing = 0;
     float set_font_scale = 1;
     int set_hinting = 0;
-    int set_force_override = 0;
     if (opts->ass_style_override) {
         set_use_margins = opts->ass_use_margins;
-#if LIBASS_VERSION >= 0x01010000
         set_sub_pos = 100 - opts->sub_pos;
-#endif
         set_line_spacing = opts->ass_line_spacing;
         set_hinting = opts->ass_hinting;
-        set_force_override = opts->ass_style_override == 3;
         set_font_scale = opts->sub_scale;
         if (opts->sub_scale_with_window) {
             int vidh = dim->h - (dim->mt + dim->mb);
             set_font_scale *= dim->h / (float)MPMAX(vidh, 1);
         }
+        if (!opts->sub_scale_by_window) {
+            double factor = dim->h / 720.0;
+            if (factor != 0.0)
+                set_font_scale /= factor;
+        }
     }
 
     ass_set_use_margins(priv, set_use_margins);
@@ -161,8 +160,13 @@ void mp_ass_configure(ASS_Renderer *priv, struct MPOpts *opts,
 #if LIBASS_VERSION >= 0x01000000
     ass_set_shaper(priv, opts->ass_shaper);
 #endif
-#if LIBASS_VERSION >= 0x01103000
-    ass_set_selective_style_override_enabled(priv, set_force_override);
+#if LIBASS_VERSION >= 0x01103001
+    int set_force_flags = 0;
+    if (opts->ass_style_override == 3)
+        set_force_flags |= ASS_OVERRIDE_BIT_STYLE | ASS_OVERRIDE_BIT_FONT_SIZE;
+    if (opts->ass_style_override == 4)
+        set_force_flags |= ASS_OVERRIDE_BIT_FONT_SIZE;
+    ass_set_selective_style_override_enabled(priv, set_force_flags);
     ASS_Style style = {0};
     mp_ass_set_style(&style, 288, opts->sub_text_style);
     ass_set_selective_style_override(priv, &style);
@@ -177,7 +181,7 @@ void mp_ass_configure_fonts(ASS_Renderer *priv, struct osd_style_opts *opts,
                             struct mpv_global *global, struct mp_log *log)
 {
     void *tmp = talloc_new(NULL);
-    char *default_font = mp_find_user_config_file(tmp, global, "subfont.ttf");
+    char *default_font = mp_find_config_file(tmp, global, "subfont.ttf");
     char *config       = mp_find_config_file(tmp, global, "fonts.conf");
 
     if (default_font && !mp_path_exists(default_font))
@@ -203,7 +207,7 @@ void mp_ass_render_frame(ASS_Renderer *renderer, ASS_Track *track, double time,
 
     res->parts = *parts;
     res->num_parts = 0;
-    int num_parts_alloc = MP_TALLOC_ELEMS(res->parts);
+    int num_parts_alloc = MP_TALLOC_AVAIL(res->parts);
     for (struct ass_image *img = imgs; img; img = img->next) {
         if (img->w == 0 || img->h == 0)
             continue;
@@ -249,7 +253,7 @@ static void message_callback(int level, const char *format, va_list va, void *ct
 
 ASS_Library *mp_ass_init(struct mpv_global *global, struct mp_log *log)
 {
-    char *path = mp_find_user_config_file(NULL, global, "fonts");
+    char *path = mp_find_config_file(NULL, global, "fonts");
     ASS_Library *priv = ass_library_init();
     if (!priv)
         abort();
