@@ -48,9 +48,6 @@ static bool create_dc(struct MPGLContext *ctx, int flags)
     pfd.nVersion = 1;
     pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
 
-    if (flags & VOFLAG_STEREO)
-        pfd.dwFlags |= PFD_STEREO;
-
     pfd.iPixelType = PFD_TYPE_RGBA;
     pfd.cColorBits = 24;
     pfd.iLayerType = PFD_MAIN_PLANE;
@@ -202,8 +199,7 @@ static void create_ctx(void *ptr)
     if (!create_dc(ctx, w32_ctx->flags))
         return;
 
-    if (ctx->requested_gl_version >= MPGL_VER(3, 0))
-        create_context_w32_gl3(ctx);
+    create_context_w32_gl3(ctx);
     if (!w32_ctx->context)
         create_context_w32_old(ctx);
     wglMakeCurrent(w32_ctx->hdc, NULL);
