@@ -32,6 +32,7 @@ struct glx_context {
     XVisualInfo *vinfo;
     GLXContext context;
     GLXFBConfig fbc;
+    bool force_es;
 };
 
 static bool create_context_x11_old(struct MPGLContext *ctx)
@@ -68,16 +69,14 @@ static bool create_context_x11_old(struct MPGLContext *ctx)
 
     glx_ctx->context = new_context;
 
-    if (!glXIsDirect(vo->x11->display, new_context))
-        ctx->gl->mpgl_caps &= ~MPGL_CAP_NO_SW;
-
     return true;
 }
 
 typedef GLXContext (*glXCreateContextAttribsARBProc)
     (Display*, GLXFBConfig, GLXContext, Bool, const int*);
 
-static bool create_context_x11_gl3(struct MPGLContext *ctx, bool debug)
+static bool create_context_x11_gl3(struct MPGLContext *ctx, int vo_flags,
+                                   int gl_version, bool es)
 {
     struct glx_context *glx_ctx = ctx->priv;
     struct vo *vo = ctx->vo;
@@ -97,12 +96,20 @@ static bool create_context_x11_gl3(struct MPGLContext *ctx, bool debug)
         return false;
     }
 
-    int gl_version = ctx->requested_gl_version;
+    int ctx_flags = vo_flags & VOFLAG_GL_DEBUG ? GLX_CONTEXT_DEBUG_BIT_ARB : 0;
+    int profile_mask = GLX_CONTEXT_CORE_PROFILE_BIT_ARB;
+
+    if (es) {
+        profile_mask = GLX_CONTEXT_ES2_PROFILE_BIT_EXT;
+        if (!(glxstr && strstr(glxstr, "GLX_EXT_create_context_es2_profile")))
+            return false;
+    }
+
     int context_attribs[] = {
         GLX_CONTEXT_MAJOR_VERSION_ARB, MPGL_VER_GET_MAJOR(gl_version),
         GLX_CONTEXT_MINOR_VERSION_ARB, MPGL_VER_GET_MINOR(gl_version),
-        GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
-        GLX_CONTEXT_FLAGS_ARB, debug ? GLX_CONTEXT_DEBUG_BIT_ARB : 0,
+        GLX_CONTEXT_PROFILE_MASK_ARB, profile_mask,
+        GLX_CONTEXT_FLAGS_ARB, ctx_flags,
         None
     };
     GLXContext context = glXCreateContextAttribsARB(vo->x11->display,
@@ -124,9 +131,6 @@ static bool create_context_x11_gl3(struct MPGLContext *ctx, bool debug)
 
     mpgl_load_functions(ctx->gl, (void *)glXGetProcAddressARB, glxstr, vo->log);
 
-    if (!glXIsDirect(vo->x11->display, context))
-        ctx->gl->mpgl_caps &= ~MPGL_CAP_NO_SW;
-
     return true;
 }
 
@@ -192,18 +196,17 @@ static bool config_window_x11(struct MPGLContext *ctx, int flags)
 
     int glx_major, glx_minor;
 
-    // FBConfigs were added in GLX version 1.3.
     if (!glXQueryVersion(vo->x11->display, &glx_major, &glx_minor)) {
         MP_ERR(vo, "GLX not found.\n");
         return false;
     }
+    // FBConfigs were added in GLX version 1.3.
     if (MPGL_VER(glx_major, glx_minor) <  MPGL_VER(1, 3)) {
         MP_ERR(vo, "GLX version older than 1.3.\n");
         return false;
     }
 
     int glx_attribs[] = {
-        GLX_STEREO, False,
         GLX_X_RENDERABLE, True,
         GLX_X_VISUAL_TYPE, GLX_TRUE_COLOR,
         GLX_RED_SIZE, 1,
@@ -222,16 +225,6 @@ static bool config_window_x11(struct MPGLContext *ctx, int flags)
             flags &= ~VOFLAG_ALPHA;
         }
     }
-    if (flags & VOFLAG_STEREO) {
-        set_glx_attrib(glx_attribs, GLX_STEREO, True);
-        fbc = select_fb_config(vo, glx_attribs, flags);
-        if (!fbc) {
-            MP_ERR(vo, "Could not find a stereo visual,"
-                       " 3D will probably not work!\n");
-            set_glx_attrib(glx_attribs, GLX_STEREO, False);
-            flags &= ~VOFLAG_STEREO;
-        }
-    }
     if (!fbc)
         fbc = select_fb_config(vo, glx_attribs, flags);
     if (!fbc) {
@@ -255,19 +248,20 @@ static bool config_window_x11(struct MPGLContext *ctx, int flags)
 
     vo_x11_config_vo_window(vo, glx_ctx->vinfo, flags, "gl");
 
+    int gl_version = ctx->requested_gl_version;
     bool success = false;
-    if (ctx->requested_gl_version >= MPGL_VER(3, 0))
-        success = create_context_x11_gl3(ctx, flags & VOFLAG_GL_DEBUG);
-    if (!success)
-        success = create_context_x11_old(ctx);
+    if (!glx_ctx->force_es) {
+        success = create_context_x11_gl3(ctx, flags, gl_version, false);
+        if (!success)
+            success = create_context_x11_old(ctx);
+    }
+    if (!success) // try ES
+        success = create_context_x11_gl3(ctx, flags, 200, true);
+    if (success && !glXIsDirect(vo->x11->display, glx_ctx->context))
+        ctx->gl->mpgl_caps |= MPGL_CAP_SW;
     return success;
 }
 
-
-/**
- * \brief free the VisualInfo and GLXContext of an OpenGL context.
- * \ingroup glcontext
- */
 static void releaseGlContext_x11(MPGLContext *ctx)
 {
     struct glx_context *glx_ctx = ctx->priv;
@@ -299,3 +293,10 @@ void mpgl_set_backend_x11(MPGLContext *ctx)
     ctx->vo_uninit = vo_x11_uninit;
     ctx->vo_control = vo_x11_control;
 }
+
+void mpgl_set_backend_x11es(MPGLContext *ctx)
+{
+    mpgl_set_backend_x11(ctx);
+    struct glx_context *glx_ctx = ctx->priv;
+    glx_ctx->force_es = true;
+}
