@@ -30,17 +30,23 @@
 #include "common/common.h"
 #include "options/options.h"
 
+// VO needs to redraw
 #define VO_EVENT_EXPOSE 1
+// VO needs to update state to a new window size
 #define VO_EVENT_RESIZE 2
-#define VO_EVENT_ICC_PROFILE_PATH_CHANGED 4
+// The ICC profile needs to be reloaded
+#define VO_EVENT_ICC_PROFILE_CHANGED 4
+// Some other window state changed
+#define VO_EVENT_WIN_STATE 8
+
+// Set of events the player core may be interested in.
+#define VO_EVENTS_USER (VO_EVENT_RESIZE | VO_EVENT_WIN_STATE)
 
 enum mp_voctrl {
     /* signal a device reset seek */
     VOCTRL_RESET = 1,
     /* Handle input and redraw events, called by vo_check_events() */
     VOCTRL_CHECK_EVENTS,
-    /* used to switch to fullscreen */
-    VOCTRL_FULLSCREEN,
     /* signal a device pause */
     VOCTRL_PAUSE,
     /* start/resume playback */
@@ -57,11 +63,14 @@ enum mp_voctrl {
 
     // Redraw the image previously passed to draw_image() (basically, repeat
     // the previous draw_image call). If this is handled, the OSD should also
-    // be updated and redrawn.
+    // be updated and redrawn. Optional; emulated if not available.
     VOCTRL_REDRAW_FRAME,
 
+    VOCTRL_FULLSCREEN,
     VOCTRL_ONTOP,
     VOCTRL_BORDER,
+    VOCTRL_ALL_WORKSPACES,
+
     VOCTRL_UPDATE_WINDOW_TITLE,         // char*
 
     VOCTRL_SET_CURSOR_VISIBILITY,       // bool*
@@ -77,16 +86,23 @@ enum mp_voctrl {
     VOCTRL_GET_UNFS_WINDOW_SIZE,        // int[2] (w/h)
     VOCTRL_SET_UNFS_WINDOW_SIZE,        // int[2] (w/h)
 
+    VOCTRL_GET_WIN_STATE,               // int* (VO_WIN_STATE_* flags)
+
+    // char *** (NULL terminated array compatible with CONF_TYPE_STRING_LIST)
+    // names for displays the window is on
+    VOCTRL_GET_DISPLAY_NAMES,
+
     // The VO is supposed to set  "known" fields, and leave the others
     // untouched or set to 0.
     // imgfmt/w/h/d_w/d_h can be omitted for convenience.
     VOCTRL_GET_COLORSPACE,              // struct mp_image_params*
 
-    VOCTRL_SCREENSHOT,                  // struct voctrl_screenshot_args*
+    // Retrieve window contents. (Normal screenshots use vo_get_current_frame().)
+    VOCTRL_SCREENSHOT_WIN,              // struct mp_image**
 
     VOCTRL_SET_COMMAND_LINE,            // char**
 
-    VOCTRL_GET_ICC_PROFILE_PATH,        // char**
+    VOCTRL_GET_ICC_PROFILE,             // bstr*
     VOCTRL_GET_DISPLAY_FPS,             // double*
     VOCTRL_GET_RECENT_FLIP_TIME,        // int64_t* (using mp_time_us())
 
@@ -105,24 +121,8 @@ struct voctrl_get_equalizer_args {
     int *valueptr;
 };
 
-// VOCTRL_SCREENSHOT
-struct voctrl_screenshot_args {
-    // 0: Save image of the currently displayed video frame, in original
-    //    resolution.
-    // 1: Save full screenshot of the window. Should contain OSD, EOSD, and the
-    //    scaled video.
-    // The value of this variable can be ignored if only a single method is
-    // implemented.
-    int full_window;
-    // Will be set to a newly allocated image, that contains the screenshot.
-    // The caller has to free the image with talloc_free().
-    // It is not specified whether the image data is a copy or references the
-    // image data directly.
-    // Can be NULL on failure.
-    struct mp_image *out_image;
-    // Whether the VO rendered OSD/subtitles into out_image
-    bool has_osd;
-};
+// VOCTRL_GET_WIN_STATE
+#define VO_WIN_STATE_MINIMIZED 1
 
 #define VO_TRUE         true
 #define VO_FALSE        false
@@ -132,7 +132,6 @@ struct voctrl_screenshot_args {
 
 #define VOFLAG_FLIPPING         0x08
 #define VOFLAG_HIDDEN           0x10  //< Use to create a hidden window
-#define VOFLAG_STEREO           0x20  //< Use to create a stereo-capable window
 #define VOFLAG_GL_DEBUG         0x40  // Hint to request debug OpenGL context
 #define VOFLAG_ALPHA            0x80  // Hint to request alpha framebuffer
 
@@ -146,6 +145,19 @@ struct osd_state;
 struct mp_image;
 struct mp_image_params;
 
+struct vo_extra {
+    struct input_ctx *input_ctx;
+    struct osd_state *osd;
+    struct encode_lavc_context *encode_lavc_ctx;
+    struct mpv_opengl_cb_context *opengl_cb_context;
+};
+
+struct frame_timing {
+    int64_t pts;
+    int64_t next_vsync;
+    int64_t prev_vsync;
+};
+
 struct vo_driver {
     // Encoding functionality, which can be invoked via --o only.
     bool encode;
@@ -153,7 +165,7 @@ struct vo_driver {
     // VO_CAP_* bits
     int caps;
 
-    // Disable video timing, push frames as quickly as possible.
+    // Disable video timing, push frames as quickly as possible, never redraw.
     bool untimed;
 
     const char *name;
@@ -167,9 +179,9 @@ struct vo_driver {
     /*
      * Whether the given image format is supported and config() will succeed.
      * format: one of IMGFMT_*
-     * returns: 0 on not supported, otherwise a bitmask of VFCAP_* values
+     * returns: 0 on not supported, otherwise 1
      */
-    int (*query_format)(struct vo *vo, uint32_t format);
+    int (*query_format)(struct vo *vo, int format);
 
     /*
      * Initialize or reconfigure the display driver.
@@ -193,6 +205,12 @@ struct vo_driver {
      */
     void (*draw_image)(struct vo *vo, struct mp_image *mpi);
 
+    /* Like draw image, but is called before every vsync with timing
+     * information
+     */
+    void (*draw_image_timed)(struct vo *vo, struct mp_image *mpi,
+                             struct frame_timing *t);
+
     /*
      * Blit/Flip buffer to the screen. Must be called after each frame!
      */
@@ -254,6 +272,7 @@ struct vo {
     struct encode_lavc_context *encode_lavc_ctx;
     struct vo_internal *in;
     struct mp_vo_opts *opts;
+    struct vo_extra extra;
 
     // --- The following fields are generally only changed during initialization.
 
@@ -278,10 +297,7 @@ struct vo {
 };
 
 struct mpv_global;
-struct vo *init_best_video_out(struct mpv_global *global,
-                               struct input_ctx *input_ctx,
-                               struct osd_state *osd,
-                               struct encode_lavc_context *encode_lavc_ctx);
+struct vo *init_best_video_out(struct mpv_global *global, struct vo_extra *ex);
 int vo_reconfig(struct vo *vo, struct mp_image_params *p, int flags);
 
 int vo_control(struct vo *vo, uint32_t request, void *data);
@@ -297,9 +313,13 @@ void vo_seek_reset(struct vo *vo);
 void vo_destroy(struct vo *vo);
 void vo_set_paused(struct vo *vo, bool paused);
 int64_t vo_get_drop_count(struct vo *vo);
-int vo_query_format(struct vo *vo, int format);
+void vo_increment_drop_count(struct vo *vo, int64_t n);
+void vo_query_formats(struct vo *vo, uint8_t *list);
+void vo_event(struct vo *vo, int event);
+int vo_query_and_reset_events(struct vo *vo, int events);
+struct mp_image *vo_get_current_frame(struct vo *vo);
 
-void vo_set_flip_queue_offset(struct vo *vo, int64_t us);
+void vo_set_flip_queue_params(struct vo *vo, int64_t offset_us, bool vsync_timed);
 int64_t vo_get_vsync_interval(struct vo *vo);
 void vo_wakeup(struct vo *vo);
 
