@@ -26,7 +26,6 @@
 #include "options/options.h"
 #include "video/fmt-conversion.h"
 #include "video/mp_image.h"
-#include "video/vfcap.h"
 #include "talloc.h"
 #include "vo.h"
 
@@ -56,6 +55,8 @@ struct priv {
     int worst_time_base_is_stream;
 
     struct mp_image_params real_colorspace;
+
+    bool shutdown;
 };
 
 static int preinit(struct vo *vo)
@@ -75,7 +76,7 @@ static void draw_image_unlocked(struct vo *vo, mp_image_t *mpi);
 static void uninit(struct vo *vo)
 {
     struct priv *vc = vo->priv;
-    if (!vc)
+    if (!vc || vc->shutdown)
         return;
 
     pthread_mutex_lock(&vo->encode_lavc_ctx->lock);
@@ -86,6 +87,8 @@ static void uninit(struct vo *vo)
     mp_image_unrefp(&vc->lastimg);
 
     pthread_mutex_unlock(&vo->encode_lavc_ctx->lock);
+
+    vc->shutdown = true;
 }
 
 static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
@@ -97,7 +100,7 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
     uint32_t width = params->w;
     uint32_t height = params->h;
 
-    if (!vc)
+    if (!vc || vc->shutdown)
         return -1;
 
     pthread_mutex_lock(&vo->encode_lavc_ctx->lock);
@@ -132,6 +135,14 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
         goto error;
     }
 
+    // When we get here, this must be the first call to reconfigure(). Thus, we
+    // can rely on no existing data in vc having been allocated yet.
+    // Reason:
+    // - Second calls after reconfigure() already failed once fail (due to the
+    //   vc->shutdown check above).
+    // - Second calls after reconfigure() already succeeded once return early
+    //   (due to the vc->stream check above).
+
     vc->lastipts = AV_NOPTS_VALUE;
     vc->lastframeipts = AV_NOPTS_VALUE;
     vc->lastencodedipts = AV_NOPTS_VALUE;
@@ -169,19 +180,17 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
 
     vc->buffer = talloc_size(vc, vc->buffer_size);
 
-    mp_image_unrefp(&vc->lastimg);
-
 done:
     pthread_mutex_unlock(&vo->encode_lavc_ctx->lock);
     return 0;
 
 error:
     pthread_mutex_unlock(&vo->encode_lavc_ctx->lock);
-    uninit(vo);
+    vc->shutdown = true;
     return -1;
 }
 
-static int query_format(struct vo *vo, uint32_t format)
+static int query_format(struct vo *vo, int format)
 {
     enum AVPixelFormat pix_fmt = imgfmt2pixfmt(format);
 
@@ -191,7 +200,7 @@ static int query_format(struct vo *vo, uint32_t format)
     pthread_mutex_lock(&vo->encode_lavc_ctx->lock);
     int flags = 0;
     if (encode_lavc_supports_pixfmt(vo->encode_lavc_ctx, pix_fmt))
-        flags = VFCAP_CSP_SUPPORTED | VFCAP_CSP_SUPPORTED_BY_HW;
+        flags = 1;
     pthread_mutex_unlock(&vo->encode_lavc_ctx->lock);
     return flags;
 }
@@ -288,7 +297,7 @@ static void draw_image_unlocked(struct vo *vo, mp_image_t *mpi)
 
     double pts = mpi ? mpi->pts : MP_NOPTS_VALUE;
 
-    if (!vc)
+    if (!vc || vc->shutdown)
         goto done;
     if (!encode_lavc_start(ectx)) {
         MP_WARN(vo, "NOTE: skipped initial video frame (probably because audio is not there yet)\n");
@@ -408,7 +417,7 @@ static void draw_image_unlocked(struct vo *vo, mp_image_t *mpi)
     if (vc->lastipts != AV_NOPTS_VALUE) {
 
         // we have a valid image in lastimg
-        while (vc->lastipts < frameipts) {
+        while (vc->lastimg && vc->lastipts < frameipts) {
             int64_t thisduration = vc->harddup ? 1 : (frameipts - vc->lastipts);
             AVPacket packet;
 
