@@ -37,7 +37,6 @@
 #include "sub/img_convert.h"
 #include "x11_common.h"
 
-#include "video/vfcap.h"
 #include "video/mp_image.h"
 #include "video/vaapi.h"
 #include "video/hwdec.h"
@@ -175,11 +174,11 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
     return 0;
 }
 
-static int query_format(struct vo *vo, uint32_t imgfmt)
+static int query_format(struct vo *vo, int imgfmt)
 {
     struct priv *p = vo->priv;
-    if (imgfmt == IMGFMT_VAAPI || va_image_format_from_imgfmt(p->va_image_formats, imgfmt))
-        return VFCAP_CSP_SUPPORTED | VFCAP_CSP_SUPPORTED_BY_HW;
+    if (imgfmt == IMGFMT_VAAPI || va_image_format_from_imgfmt(p->mpvaapi, imgfmt))
+        return 1;
 
     return 0;
 }
@@ -299,21 +298,6 @@ static void draw_image(struct vo *vo, struct mp_image *mpi)
     draw_osd(vo);
 }
 
-static struct mp_image *get_screenshot(struct priv *p)
-{
-    struct mp_image *hwimg = p->output_surfaces[p->visible_surface];
-    if (!hwimg)
-        return NULL;
-    struct mp_image *img = va_surface_download(hwimg, NULL);
-    if (!img)
-        return NULL;
-    struct mp_image_params params = p->image_params;
-    params.imgfmt = img->imgfmt;
-    mp_image_params_guess_csp(&params); // ensure colorspace consistency
-    mp_image_set_params(img, &params);
-    return img;
-}
-
 static void free_subpicture(struct priv *p, struct vaapi_osd_image *img)
 {
     if (img->image.image_id != VA_INVALID_ID)
@@ -382,7 +366,7 @@ static void draw_osd_cb(void *pctx, struct sub_bitmaps *imgs)
 
         struct vaapi_osd_image *img = &part->image;
         struct mp_image vaimg;
-        if (va_image_map(p->mpvaapi, &img->image, &vaimg) < 0)
+        if (!va_image_map(p->mpvaapi, &img->image, &vaimg))
             goto error;
 
         // Clear borders and regions uncovered by sub-bitmaps
@@ -403,7 +387,7 @@ static void draw_osd_cb(void *pctx, struct sub_bitmaps *imgs)
                        vaimg.stride[0], sub->stride);
         }
 
-        if (va_image_unmap(p->mpvaapi, &img->image) < 0)
+        if (!va_image_unmap(p->mpvaapi, &img->image))
             goto error;
 
         part->subpic = (struct vaapi_subpic) {
@@ -549,11 +533,6 @@ static int control(struct vo *vo, uint32_t request, void *data)
         p->output_surface = p->visible_surface;
         draw_osd(vo);
         return true;
-    case VOCTRL_SCREENSHOT: {
-        struct voctrl_screenshot_args *args = data;
-        args->out_image = get_screenshot(p);
-        return true;
-    }
     case VOCTRL_GET_PANSCAN:
         return VO_TRUE;
     case VOCTRL_SET_PANSCAN:
@@ -567,6 +546,7 @@ static int control(struct vo *vo, uint32_t request, void *data)
         resize(p);
     if (events & VO_EVENT_EXPOSE)
         vo->want_redraw = true;
+    vo_event(vo, events);
     return r;
 }
 
@@ -609,7 +589,7 @@ static int preinit(struct vo *vo)
         goto fail;
     }
 
-    p->hwdec_info.vaapi_ctx = p->mpvaapi;
+    p->hwdec_info.hwctx = &p->mpvaapi->hwctx;
 
     if (va_guess_if_emulated(p->mpvaapi)) {
         MP_WARN(vo, "VA-API is most likely emulated via VDPAU.\n"
