@@ -46,7 +46,6 @@
 #include "video/csputils.h"
 #include "sub/osd.h"
 #include "options/m_option.h"
-#include "video/vfcap.h"
 #include "video/mp_image.h"
 #include "osdep/timer.h"
 #include "bitmap_packer.h"
@@ -81,7 +80,6 @@ struct vdpctx {
     VdpPresentationQueue               flip_queue;
 
     VdpOutputSurface                   output_surfaces[MAX_OUTPUT_SURFACES];
-    VdpOutputSurface                   screenshot_surface;
     int                                num_output_surfaces;
     VdpOutputSurface                   black_pixel;
 
@@ -99,7 +97,7 @@ struct vdpctx {
     int                                chroma_deint;
     int                                flip_offset_window;
     int                                flip_offset_fs;
-    int                                flip_offset_ms;
+    int64_t                            flip_offset_us;
     bool                               flip;
 
     VdpRect                            src_rect_vid;
@@ -177,24 +175,16 @@ static int render_video_to_output_surface(struct vo *vo,
     }
 
     if (vc->rgb_mode) {
-        VdpOutputSurface surface = (uintptr_t)mpi->planes[3];
+        // Clear the borders between video and window (if there are any).
+        // For some reason, video_mixer_render doesn't need it for YUV.
         int flags = VDP_OUTPUT_SURFACE_RENDER_ROTATE_0;
         vdp_st = vdp->output_surface_render_output_surface(output_surface,
                                                            NULL, vc->black_pixel,
                                                            NULL, NULL, NULL,
                                                            flags);
         CHECK_VDP_WARNING(vo, "Error clearing screen");
-        vdp_st = vdp->output_surface_render_output_surface(output_surface,
-                                                           output_rect,
-                                                           surface,
-                                                           video_rect,
-                                                           NULL, NULL, flags);
-        CHECK_VDP_WARNING(vo, "Error when calling "
-                          "vdp_output_surface_render_output_surface");
-        return 0;
     }
 
-
     struct mp_vdpau_mixer_frame *frame = mp_vdpau_mixed_frame_get(mpi);
     struct mp_vdpau_mixer_opts opts = {0};
     if (frame)
@@ -258,11 +248,10 @@ static void resize(struct vo *vo)
     vc->src_rect_vid.y0 = vc->flip ? src_rect.y1 : src_rect.y0;
     vc->src_rect_vid.y1 = vc->flip ? src_rect.y0 : src_rect.y1;
 
-    vc->flip_offset_ms = vo->opts->fullscreen ?
-                         vc->flip_offset_fs :
-                         vc->flip_offset_window;
-
-    vo_set_flip_queue_offset(vo, vc->flip_offset_ms * 1000);
+    vc->flip_offset_us = vo->opts->fullscreen ?
+                         1000LL * vc->flip_offset_fs :
+                         1000LL * vc->flip_offset_window;
+    vo_set_flip_queue_params(vo, vc->flip_offset_us, false);
 
     if (vc->output_surface_width < vo->dwidth
         || vc->output_surface_height < vo->dheight) {
@@ -346,12 +335,6 @@ static void free_video_specific(struct vo *vo)
 
     forget_frames(vo, false);
 
-    if (vc->screenshot_surface != VDP_INVALID_HANDLE) {
-        vdp_st = vdp->output_surface_destroy(vc->screenshot_surface);
-        CHECK_VDP_WARNING(vo, "Error when calling vdp_output_surface_destroy");
-    }
-    vc->screenshot_surface = VDP_INVALID_HANDLE;
-
     if (vc->black_pixel != VDP_INVALID_HANDLE) {
         vdp_st = vdp->output_surface_destroy(vc->black_pixel);
         CHECK_VDP_WARNING(vo, "Error when calling vdp_output_surface_destroy");
@@ -401,7 +384,6 @@ static void mark_vdpau_objects_uninitialized(struct vo *vo)
     vc->flip_target = VDP_INVALID_HANDLE;
     for (int i = 0; i < MAX_OUTPUT_SURFACES; i++)
         vc->output_surfaces[i] = VDP_INVALID_HANDLE;
-    vc->screenshot_surface = VDP_INVALID_HANDLE;
     vc->vdp_device = VDP_INVALID_HANDLE;
     for (int i = 0; i < MAX_OSD_PARTS; i++) {
         struct osd_bitmap_surface *sfc = &vc->osd_surfaces[i];
@@ -752,11 +734,11 @@ static int flip_page_timed(struct vo *vo, int64_t pts_us, int duration)
     /* This should normally never happen.
      * - The last queued frame can't have a PTS that goes more than 50ms in the
      *   future. This is guaranteed by vo.c, which currently actually queues
-     *   ahead by roughly 50ms, plus the flip queue offset. Just to be sure
+     *   ahead by roughly the flip queue offset. Just to be sure
      *   give some additional room by doubling the time.
      * - The last vsync can never be in the future.
      */
-    int64_t max_pts_ahead = (vc->flip_offset_ms + 50) * 1000 * 1000 * 2;
+    int64_t max_pts_ahead = vc->flip_offset_us * 1000 * 2;
     if (vc->last_queue_time > now + max_pts_ahead ||
         vc->recent_vsync_time > now)
     {
@@ -837,11 +819,8 @@ static void draw_image(struct vo *vo, struct mp_image *mpi)
     check_preemption(vo);
 
     struct mp_image *vdp_mpi = mp_vdpau_upload_video_surface(vc->mpvdp, mpi);
-    if (vdp_mpi) {
-        mp_image_copy_attributes(vdp_mpi, mpi);
-    } else {
+    if (!vdp_mpi)
         MP_ERR(vo, "Could not upload image.\n");
-    }
     talloc_free(mpi);
 
     talloc_free(vc->current_image);
@@ -880,30 +859,6 @@ static struct mp_image *read_output_surface(struct vo *vo,
     return image;
 }
 
-static struct mp_image *get_screenshot(struct vo *vo)
-{
-    struct vdpctx *vc = vo->priv;
-    VdpStatus vdp_st;
-    struct vdp_functions *vdp = vc->vdp;
-
-    if (!vo->params)
-        return NULL;
-
-    if (vc->screenshot_surface == VDP_INVALID_HANDLE) {
-        vdp_st = vdp->output_surface_create(vc->vdp_device,
-                                            OUTPUT_RGBA_FORMAT,
-                                            vo->params->d_w, vo->params->d_h,
-                                            &vc->screenshot_surface);
-        CHECK_VDP_WARNING(vo, "Error when calling vdp_output_surface_create");
-    }
-
-    VdpRect in = { .x1 = vo->params->w, .y1 = vo->params->h };
-    VdpRect out = { .x1 = vo->params->d_w, .y1 = vo->params->d_h };
-    render_video_to_output_surface(vo, vc->screenshot_surface, &out, &in);
-
-    return read_output_surface(vo, vc->screenshot_surface, out.x1, out.y1);
-}
-
 static struct mp_image *get_window_screenshot(struct vo *vo)
 {
     struct vdpctx *vc = vo->priv;
@@ -916,15 +871,14 @@ static struct mp_image *get_window_screenshot(struct vo *vo)
     return image;
 }
 
-static int query_format(struct vo *vo, uint32_t format)
+static int query_format(struct vo *vo, int format)
 {
     struct vdpctx *vc = vo->priv;
 
-    int flags = VFCAP_CSP_SUPPORTED | VFCAP_CSP_SUPPORTED_BY_HW;
     if (mp_vdpau_get_format(format, NULL, NULL))
-        return flags;
+        return 1;
     if (!vc->force_yuv && mp_vdpau_get_rgb_format(format, NULL))
-        return flags;
+        return 1;
     return 0;
 }
 
@@ -985,13 +939,13 @@ static int preinit(struct vo *vo)
     if (!vo_x11_init(vo))
         return -1;
 
-    vc->mpvdp = mp_vdpau_create_device_x11(vo->log, vo->x11);
+    vc->mpvdp = mp_vdpau_create_device_x11(vo->log, vo->x11->display);
     if (!vc->mpvdp) {
         vo_x11_uninit(vo);
         return -1;
     }
 
-    vc->hwdec_info.vdpau_ctx = vc->mpvdp;
+    vc->hwdec_info.hwctx = &vc->mpvdp->hwctx;
 
     vc->video_mixer = mp_vdpau_mixer_create(vc->mpvdp, vo->log);
 
@@ -1052,10 +1006,6 @@ static int control(struct vo *vo, uint32_t request, void *data)
     check_preemption(vo);
 
     switch (request) {
-    case VOCTRL_PAUSE:
-        if (vc->dropped_frame)
-            vo->want_redraw = true;
-        return true;
     case VOCTRL_GET_HWDEC_INFO: {
         struct mp_hwdec_info **arg = data;
         *arg = &vc->hwdec_info;
@@ -1091,16 +1041,11 @@ static int control(struct vo *vo, uint32_t request, void *data)
     case VOCTRL_RESET:
         forget_frames(vo, true);
         return true;
-    case VOCTRL_SCREENSHOT: {
+    case VOCTRL_SCREENSHOT_WIN:
         if (!status_ok(vo))
             return false;
-        struct voctrl_screenshot_args *args = data;
-        if (args->full_window)
-            args->out_image = get_window_screenshot(vo);
-        else
-            args->out_image = get_screenshot(vo);
+        *(struct mp_image **)data = get_window_screenshot(vo);
         return true;
-    }
     case VOCTRL_GET_PREF_DEINT:
         *(int *)data = vc->deint;
         return true;
