@@ -41,6 +41,9 @@ struct vo_wayland_output {
     uint32_t flags;
     int32_t width;
     int32_t height;
+    int32_t refresh_rate; // fps (mHz)
+    const char *make;
+    const char *model;
     struct wl_list link;
 };
 
@@ -72,11 +75,13 @@ struct vo_wayland_state {
 
         struct wl_list output_list;
         struct wl_output *fs_output; /* fullscreen output */
-        int output_mode_received;
+        struct vo_wayland_output *current_output;
 
         int display_fd;
 
         struct wl_shm *shm;
+
+        struct wl_subcompositor *subcomp;
     } display;
 
     struct {
@@ -90,12 +95,11 @@ struct vo_wayland_state {
         int32_t sh_y;
         float aspect;
 
-        bool is_init;       // true if the window has a valid size
         bool is_fullscreen; // don't keep aspect ratio in fullscreen mode
         int32_t fs_width;   // fullscreen sizes
         int32_t fs_height;
 
-        struct wl_surface *surface;
+        struct wl_surface *video_surface;
         int32_t mouse_x; // mouse position inside the surface
         int32_t mouse_y;
         struct wl_shell_surface *shell_surface;
@@ -123,12 +127,17 @@ struct vo_wayland_state {
             struct xkb_keymap *keymap;
             struct xkb_state *state;
         } xkb;
+
+        struct wl_data_device_manager *devman;
+        struct wl_data_device *datadev;
+        struct wl_data_offer *offer;
+        int dnd_fd;
     } input;
 };
 
 int vo_wayland_init(struct vo *vo);
 void vo_wayland_uninit(struct vo *vo);
-bool vo_wayland_config(struct vo *vo, uint32_t d_width, uint32_t d_height, uint32_t flags);
+bool vo_wayland_config(struct vo *vo, uint32_t flags);
 int vo_wayland_control(struct vo *vo, int *events, int request, void *arg);
 
 #endif /* MPLAYER_WAYLAND_COMMON_H */
