@@ -6,7 +6,6 @@
 #include "mp_image.h"
 
 struct mp_image;
-struct mp_csp_details;
 struct sws_opts;
 
 // libswscale currently requires 16 bytes alignment for row pointers and
@@ -60,6 +59,8 @@ struct vf_seteq;
 int mp_sws_set_vf_equalizer(struct mp_sws_context *sws, struct vf_seteq *eq);
 int mp_sws_get_vf_equalizer(struct mp_sws_context *sws, struct vf_seteq *eq);
 
+struct mp_image *mp_img_swap_to_native(struct mp_image *img);
+
 #endif /* MP_SWS_UTILS_H */
 
 // vim: ts=4 sw=4 et tw=80
