@@ -3,6 +3,7 @@
 
 #include <stdbool.h>
 #include <inttypes.h>
+#include <pthread.h>
 #include <va/va.h>
 #include <va/va_x11.h>
 
@@ -71,22 +72,26 @@
     (const char[]){(fcc), (fcc) >> 8u, (fcc) >> 16u, (fcc) >> 24u, 0}
 
 #include "mp_image.h"
+#include "hwdec.h"
 
 struct mp_image_pool;
 struct mp_log;
 
 struct mp_vaapi_ctx {
+    struct mp_hwdec_ctx hwctx;
     struct mp_log *log;
     VADisplay display;
     struct va_image_formats *image_formats;
+    pthread_mutex_t lock;
 };
 
-struct va_image_formats;
-
 bool check_va_status(struct mp_log *log, VAStatus status, const char *msg);
 
 #define CHECK_VA_STATUS(ctx, msg) check_va_status((ctx)->log, status, msg)
 
+#define va_lock(ctx)     pthread_mutex_lock(&(ctx)->lock)
+#define va_unlock(ctx)   pthread_mutex_unlock(&(ctx)->lock)
+
 int                      va_get_colorspace_flag(enum mp_csp csp);
 
 struct mp_vaapi_ctx     *va_initialize(VADisplay *display, struct mp_log *log);
@@ -94,7 +99,7 @@ void                     va_destroy(struct mp_vaapi_ctx *ctx);
 
 enum mp_imgfmt           va_fourcc_to_imgfmt(uint32_t fourcc);
 uint32_t                 va_fourcc_from_imgfmt(int imgfmt);
-VAImageFormat *          va_image_format_from_imgfmt(const struct va_image_formats *formats, int imgfmt);
+VAImageFormat *          va_image_format_from_imgfmt(struct mp_vaapi_ctx *ctx, int imgfmt);
 bool                     va_image_map(struct mp_vaapi_ctx *ctx, VAImage *image, struct mp_image *mpi);
 bool                     va_image_unmap(struct mp_vaapi_ctx *ctx, VAImage *image);
 
