@@ -20,6 +20,12 @@ def check_pthread_flag(ctx, dependency_identifier):
     return False
 
 def check_pthreads(ctx, dependency_identifier):
+    if ctx.dependency_satisfied('win32-internal-pthreads'):
+        h = ctx.path.find_node('osdep/win32/include').abspath()
+        # define IN_WINPTHREAD to workaround mingw stupidity (we never want it
+        # to define features specific to its own pthread stuff)
+        ctx.env.CFLAGS += ['-isystem', h, '-I', h, '-DIN_WINPTHREAD']
+        return True
     if check_pthread_flag(ctx, dependency_identifier):
         return True
     platform_cflags = {
@@ -27,7 +33,6 @@ def check_pthreads(ctx, dependency_identifier):
         'freebsd': '-D_THREAD_SAFE',
         'netbsd':  '-D_THREAD_SAFE',
         'openbsd': '-D_THREAD_SAFE',
-        'win32':   '-DPTW32_STATIC_LIB',
     }.get(ctx.env.DEST_OS, '')
     libs    = ['pthreadGC2', 'pthread']
     checkfn = check_cc(fragment=pthreads_program, cflags=platform_cflags)
@@ -45,32 +50,14 @@ def check_iconv(ctx, dependency_identifier):
     return check_libs(libs, checkfn)(ctx, dependency_identifier)
 
 def check_lua(ctx, dependency_identifier):
-    if ctx.dependency_satisfied('libquvi4'):
-        quvi_lib_storage = [ 'libquvi4' ]
-        additional_lua_test_header = '#include <quvi/quvi.h>'
-        additional_lua_test_code   = load_fragment('lua_libquvi4.c')
-    elif ctx.dependency_satisfied('libquvi9'):
-        quvi_lib_storage = [ 'libquvi9' ]
-        additional_lua_test_header = '#include <quvi.h>'
-        additional_lua_test_code   = load_fragment('lua_libquvi9.c')
-    else:
-        quvi_lib_storage = []
-        additional_lua_test_header = ''
-        additional_lua_test_code   = ''
-
-    fragment = load_fragment('lua.c').format(
-        additional_lua_test_header=additional_lua_test_header,
-        additional_lua_test_code=additional_lua_test_code)
-
     lua_versions = [
         ( '51',     'lua >= 5.1.0 lua < 5.2.0'),
         ( '51deb',  'lua5.1 >= 5.1.0'), # debian
         ( '51fbsd', 'lua-5.1 >= 5.1.0'), # FreeBSD
-        ( 'luajit', 'luajit >= 2.0.0' ),
-        # assume all our dependencies (libquvi in particular) link with 5.1
         ( '52',     'lua >= 5.2.0' ),
         ( '52deb',  'lua5.2 >= 5.2.0'), # debian
         ( '52fbsd', 'lua-5.2 >= 5.2.0'), # FreeBSD
+        ( 'luajit', 'luajit >= 2.0.0' ),
     ]
 
     if ctx.options.LUA_VER:
@@ -78,11 +65,8 @@ def check_lua(ctx, dependency_identifier):
             [lv for lv in lua_versions if lv[0] == ctx.options.LUA_VER]
 
     for lua_version, pkgconfig_query in lua_versions:
-        if compose_checks(
-            check_pkg_config(pkgconfig_query, uselib_store=lua_version),
-            check_cc(fragment=fragment,
-                     use=[lua_version] + quvi_lib_storage,
-                     execute=True))(ctx, dependency_identifier):
+        if check_pkg_config(pkgconfig_query, uselib_store=lua_version) \
+            (ctx, dependency_identifier):
             # XXX: this is a bit of a hack, ask waf developers if I can copy
             # the uselib_store to 'lua'
             ctx.mark_satisfied(lua_version)
