@@ -30,28 +30,11 @@ def m_hook(self, node):
 
 def build(ctx):
     from waflib import Task
-    import syms
 
     cls = Task.classes['cprogram']
     class cprogram(cls):
         run_str = cls.hcode + '${LAST_LINKFLAGS}'
 
-        def __str__(self):
-            tgt_str = ' '.join([a.nice_path() for a in self.outputs])
-            return 'linking -> {0}\n'.format(tgt_str)
-
-    cls = Task.classes['cshlib']
-    class cshlib(cls):
-        def __str__(self):
-            tgt_str = ' '.join([a.nice_path() for a in self.outputs])
-            return 'linking -> {0}\n'.format(tgt_str)
-
-    cls = Task.classes['compile_sym']
-    class compile_sym(cls):
-        def __str__(self):
-            tgt_str = ' '.join([a.nice_path() for a in self.outputs])
-            return 'compile_sym -> {0}\n'.format(tgt_str)
-
     cls = Task.classes['macplist']
     class macplist(cls):
         def run(self):
